/*
 *	fram.c
 *
 *	GameBoy/EMU
 *
 *	* Sat Feb 19 21:54:00 JST 2005 Naoyuki Sawa
 *	- 쐬JnB
 */
#include "app.h"

/****************************************************************************
 *	
 ****************************************************************************/

// int/*%data*/ gb_read(/*%addr*/)
asm("
	.code
	.align	1
	.global	gb_read
gb_read:
	xcmp	%r0, 0x4000		; $0000-$3FFF: 16kB ROM bank #0
	xjrlt	_gb_read_mem
	xcmp	%r0, 0x8000		; $4000-$7FFF: 16kB switchable ROM bank
	xjrlt	_gb_read_rom_bank
	xcmp	%r0, 0xa000		; $8000-$A000: 8kB Video RAM
	xjrlt	_gb_read_mem
	xcmp	%r0, 0xc000		; $A000-$C000: 8kB switchable RAM bank
	xjrlt	_gb_read_ram_bank
	xcmp	%r0, 0xff00		; $C000-$FF00: 8kB Internal RAM, Echo of 8kB Internal RAM, Sprite Attrib Memory (OAM), Empty but unusable for I/O
	xjrlt	_gb_read_mem
	xcmp	%r0, 0xff80		; $FF00-$FF7F: I/O ports, Empty but unusable for I/O
	xjrlt	_gb_read_io
	xcmp	%r0, 0xffff		; $FF80-$FFFE: Internal RAM
	xjrlt	_gb_read_mem
	;------ FALLTHRU ------		; $FFFF      : Interrupt Enable Register
_gb_read_io:
	xcall	dmgcpu_leave
	xcall.d	gb_read_io
	ld.w	%r12, %r0		; %r12 <- %addr
	xjp.d	dmgcpu_enter
	ld.w	%r1, %r10		; %r10 -> %data
	;----------------------
_gb_read_mem:
	ext	dmgmem@ah
	ext	dmgmem@al
	ld.ub	%r1, [%r0]		; %data <- [%addr]
	ret
	;----------------------
_gb_read_rom_bank:
	xld.w	%r9, [ROM_BANK]		; %data <- [ROM_BANK+%addr]
	add	%r9, %r0
	ld.ub	%r1, [%r9]		; %data <- [%addr]
	ret
	;----------------------
_gb_read_ram_bank:
	xld.w	%r9, [RAM_BANK]		; %data <- [RAM_BANK+%addr]
	add	%r9, %r0
	ld.ub	%r1, [%r9]
	ret
");

// void gb_write(/*%addr,%data*/)
asm("
	.code
	.align	1
	.global	gb_write
gb_write:
	xcmp	%r0, 0x8000		; $0000-$7FFF: 16kB ROM bank #0, 16kB switchable ROM bank
	xjrlt	_gb_write_rom_bank
	xcmp	%r0, 0xa000		; $8000-$A000: 8kB Video RAM
	xjrlt	_gb_write_mem
	xcmp	%r0, 0xc000		; $A000-$C000: 8kB switchable RAM bank
	xjrlt	_gb_write_ram_bank
	xcmp	%r0, 0xff00		; $C000-$FF00: 8kB Internal RAM, Echo of 8kB Internal RAM, Sprite Attrib Memory (OAM), Empty but unusable for I/O
	xjrlt	_gb_write_mem
	xcmp	%r0, 0xff80		; $FF00-$FF7F: I/O ports, Empty but unusable for I/O
	xjrlt	_gb_write_io
	xcmp	%r0, 0xffff		; $FF80-$FFFE: Internal RAM
	xjrlt	_gb_write_mem
	;------ FALLTHRU ------		; $FFFF      : I/O
_gb_write_io:
	xcall.d	dmgcpu_leave
	ld.w	%r12, %r0		; %r12 <- %addr
	xcall.d	gb_write_io
	ld.w	%r13, %r1		; %r13 <- %data
	xjp	dmgcpu_enter
	;----------------------
_gb_write_mem:
	ext	dmgmem@ah
	ext	dmgmem@al
	ld.b	[%r0], %r1		; [%addr] <- %data
	ret
	;----------------------
_gb_write_rom_bank:
	xcall.d	dmgcpu_leave
	ld.w	%r12, %r0		; %r12 <- %addr
	xcall.d	gb_write_rom_bank
	ld.w	%r13, %r1		; %r13 <- %data
	xjp	dmgcpu_enter
	;----------------------
_gb_write_ram_bank:
	xld.w	%r9, [RAM_BANK]		; [RAM_BANK+%addr] <- %data
	add	%r9, %r0
	ld.b	[%r9], %r1
	ret
");

